@@ -57,7 +57,7 @@ static char *(p_tbis_values[]) = {"tiny", "small", "medium", "large", "huge", "g
 static char *(p_ttym_values[]) = {"xterm", "xterm2", "dec", "netterm", "jsbterm", "pterm", "urxvt", "sgr", NULL};
 #endif
 static char *(p_ve_values[]) = {"block", "insert", "all", "onemore", "none", "NONE", NULL};
-static char *(p_wop_values[]) = {"tagfile", NULL};
+static char *(p_wop_values[]) = {"fuzzy", "tagfile", "pum", NULL};
 #ifdef FEAT_WAK
 static char *(p_wak_values[]) = {"yes", "menu", "no", NULL};
 #endif
@@ -181,7 +181,7 @@ trigger_optionsset_string(
 	    set_vim_var_string(VV_OPTION_OLDLOCAL, oldval, -1);
 	}
 	apply_autocmds(EVENT_OPTIONSET,
-		       (char_u *)get_option_fullname(opt_idx), NULL, FALSE,
+		       get_option_fullname(opt_idx), NULL, FALSE,
 		       NULL);
 	reset_v_option_vars();
     }
@@ -454,6 +454,7 @@ set_string_option_direct_in_win(
     unblock_autocmds();
 }
 
+#if defined(FEAT_PROP_POPUP) || defined(PROTO)
 /*
  * Like set_string_option_direct(), but for a buffer-local option in "buf".
  * Blocks autocommands to avoid the old curbuf becoming invalid.
@@ -477,6 +478,7 @@ set_string_option_direct_in_buf(
     curwin->w_buffer = curbuf;
     unblock_autocmds();
 }
+#endif
 
 /*
  * Set a string option to a new value, and handle the effects.
@@ -756,6 +758,9 @@ did_set_string_option(
     {
 	if (briopt_check(curwin) == FAIL)
 	    errmsg = e_invalid_argument;
+	// list setting requires a redraw
+	if (curwin->w_briopt_list)
+	    redraw_all_later(NOT_VALID);
     }
 #endif
 
@@ -1003,9 +1008,9 @@ ambw_end:
 #if defined(FEAT_GUI_GTK)
 	if (errmsg == NULL && varp == &p_tenc && gui.in_use)
 	{
-	    // GTK+ 2 uses only a single encoding, and that is UTF-8.
+	    // GTK uses only a single encoding, and that is UTF-8.
 	    if (STRCMP(p_tenc, "utf-8") != 0)
-		errmsg = N_("E617: Cannot be changed in the GTK+ 2 GUI");
+		errmsg = e_cannot_be_changed_in_gtk_GUI;
 	}
 #endif
 
@@ -1026,8 +1031,8 @@ ambw_end:
 		if (convert_setup(&input_conv, p_tenc, p_enc) == FAIL
 			|| convert_setup(&output_conv, p_enc, p_tenc) == FAIL)
 		{
-		    semsg(_("E950: Cannot convert between %s and %s"),
-			    p_tenc, p_enc);
+		    semsg(_(e_cannot_convert_between_str_and_str),
+								p_tenc, p_enc);
 		    errmsg = e_invalid_argument;
 		}
 	    }
@@ -2324,8 +2329,7 @@ ambw_end:
 # ifdef FEAT_POSTSCRIPT
 	    varp == &p_pexpr ||
 # endif
-	    FALSE
-	    )
+	    varp == &p_ccv)
     {
 	char_u	**p_opt = NULL;
 	char_u	*name;
@@ -2364,6 +2368,8 @@ ambw_end:
 	if (varp == &p_pexpr)	// 'printexpr'
 	    p_opt = &p_pexpr;
 # endif
+	if (varp == &p_ccv)	// 'charconvert'
+	    p_opt = &p_ccv;
 
 	if (p_opt != NULL)
 	{
@@ -2610,6 +2616,14 @@ ambw_end:
 	update_package_paths_in_lua();
 #endif
 
+#if defined(FEAT_LINEBREAK)
+    // Changing Formatlistpattern when briopt includes the list setting:
+    // redraw
+    if ((varp == &p_flp || varp == &(curbuf->b_p_flp))
+	    && curwin->w_briopt_list)
+	redraw_all_later(NOT_VALID);
+#endif
+
     if (curwin->w_curswant != MAXCOL
 		   && (get_option_flags(opt_idx) & (P_CURSWANT | P_RALL)) != 0)
 	curwin->w_set_curswant = TRUE;
