@@ -590,11 +590,7 @@ static char *(features[]) =
 #if defined(USE_SYSTEM) && defined(UNIX)
 	"+system()",
 #endif
-#ifdef FEAT_TAG_BINS
 	"+tag_binary",
-#else
-	"-tag_binary",
-#endif
 	"-tag_old_static",
 	"-tag_any_white",
 #ifdef FEAT_TCL
@@ -754,6 +750,162 @@ static char *(features[]) =
 
 static int included_patches[] =
 {   /* Add new patch number below this line */
+/**/
+    4567,
+/**/
+    4566,
+/**/
+    4565,
+/**/
+    4564,
+/**/
+    4563,
+/**/
+    4562,
+/**/
+    4561,
+/**/
+    4560,
+/**/
+    4559,
+/**/
+    4558,
+/**/
+    4557,
+/**/
+    4556,
+/**/
+    4555,
+/**/
+    4554,
+/**/
+    4553,
+/**/
+    4552,
+/**/
+    4551,
+/**/
+    4550,
+/**/
+    4549,
+/**/
+    4548,
+/**/
+    4547,
+/**/
+    4546,
+/**/
+    4545,
+/**/
+    4544,
+/**/
+    4543,
+/**/
+    4542,
+/**/
+    4541,
+/**/
+    4540,
+/**/
+    4539,
+/**/
+    4538,
+/**/
+    4537,
+/**/
+    4536,
+/**/
+    4535,
+/**/
+    4534,
+/**/
+    4533,
+/**/
+    4532,
+/**/
+    4531,
+/**/
+    4530,
+/**/
+    4529,
+/**/
+    4528,
+/**/
+    4527,
+/**/
+    4526,
+/**/
+    4525,
+/**/
+    4524,
+/**/
+    4523,
+/**/
+    4522,
+/**/
+    4521,
+/**/
+    4520,
+/**/
+    4519,
+/**/
+    4518,
+/**/
+    4517,
+/**/
+    4516,
+/**/
+    4515,
+/**/
+    4514,
+/**/
+    4513,
+/**/
+    4512,
+/**/
+    4511,
+/**/
+    4510,
+/**/
+    4509,
+/**/
+    4508,
+/**/
+    4507,
+/**/
+    4506,
+/**/
+    4505,
+/**/
+    4504,
+/**/
+    4503,
+/**/
+    4502,
+/**/
+    4501,
+/**/
+    4500,
+/**/
+    4499,
+/**/
+    4498,
+/**/
+    4497,
+/**/
+    4496,
+/**/
+    4495,
+/**/
+    4494,
+/**/
+    4493,
+/**/
+    4492,
+/**/
+    4491,
+/**/
+    4490,
 /**/
     4489,
 /**/
