@@ -91,6 +91,9 @@ typedef enum {
     ISN_PUSHJOB,	// push channel isn_arg.job
     ISN_NEWLIST,	// push list from stack items, size is isn_arg.number
     ISN_NEWDICT,	// push dict from stack items, size is isn_arg.number
+    ISN_NEWPARTIAL,	// push NULL partial
+
+    ISN_AUTOLOAD,	// get item from autoload import, function or variable
 
     // function call
     ISN_BCALL,	    // call builtin function isn_arg.bfunc
@@ -118,7 +121,7 @@ typedef enum {
     ISN_CATCH,	    // drop v:exception
     ISN_FINALLY,    // start of :finally block
     ISN_ENDTRY,	    // take entry off from ec_trystack
-    ISN_TRYCONT,    // handle :continue inside a :try statement
+    ISN_TRYCONT,    // handle :continue or :break inside a :try statement
 
     // more expression operations
     ISN_ADDLIST,    // add two lists
@@ -132,6 +135,7 @@ typedef enum {
     // comparative operations; isn_arg.op.op_type is exprtype_T, op_ic used
     ISN_COMPAREBOOL,
     ISN_COMPARESPECIAL,
+    ISN_COMPARENULL,
     ISN_COMPARENR,
     ISN_COMPAREFLOAT,
     ISN_COMPARESTRING,
@@ -594,6 +598,8 @@ typedef struct {
     endlabel_T	*ts_end_label;	    // jump to :finally or :endtry
     int		ts_catch_label;	    // instruction idx of last CATCH
     int		ts_caught_all;	    // "catch" without argument encountered
+    int		ts_has_finally;	    // "finally" encountered
+    int		ts_no_return;	    // one of the blocks did not end in return
 } tryscope_T;
 
 typedef enum {
@@ -687,7 +693,7 @@ typedef struct {
 } lhs_T;
 
 /*
- * Context for compiling lines of Vim script.
+ * Context for compiling lines of a :def function.
  * Stores info about the local variables and condition stack.
  */
 struct cctx_S {
@@ -706,8 +712,6 @@ struct cctx_S {
     int		ctx_has_closure;    // set to one if a closure was created in
 				    // the function
 
-    garray_T	ctx_imports;	    // imported items
-
     skip_T	ctx_skip;
     scope_T	*ctx_scope;	    // current scope, NULL at toplevel
     int		ctx_had_return;	    // last seen statement was "return"
